#!/usr/bin/env python3
import os.path
from time import sleep
from typing import Optional
from argparse import ArgumentParser, ArgumentError

from home.config import config
from home.mqtt import MqttNode, MqttWrapper, get_mqtt_modules
from home.mqtt import MqttNodesConfig

mqtt_node: Optional[MqttNode] = None
mqtt: Optional[MqttWrapper] = None


if __name__ == '__main__':
    nodes_config = MqttNodesConfig()

    parser = ArgumentParser()
    parser.add_argument('--node-id', type=str, required=True, choices=nodes_config.get_nodes(only_names=True))
    parser.add_argument('--modules', type=str, choices=get_mqtt_modules(), nargs='*',
                        help='mqtt modules to include')
    parser.add_argument('--switch-relay', choices=[0, 1], type=int,
                        help='send relay state')
    parser.add_argument('--push-ota', type=str, metavar='OTA_FILENAME',
                        help='push ota, argument receives filename')

    config.load_app(parser=parser, no_config=True)
    arg = parser.parse_args()

    if arg.switch_relay is not None and 'relay' not in arg.modules:
        raise ArgumentError(None, '--relay is only allowed when \'relay\' module included in --modules')

    mqtt = MqttWrapper(randomize_client_id=True,
                       client_id='mqtt_node_util')
    mqtt_node = MqttNode(node_id=arg.node_id,
                         node_secret=nodes_config.get_node(arg.node_id)['password'])

    mqtt.add_node(mqtt_node)

    # must-have modules
    ota_module = mqtt_node.load_module('ota')
    mqtt_node.load_module('diagnostics')

    if arg.modules:
        for m in arg.modules:
            module_instance = mqtt_node.load_module(m)
            if m == 'relay' and arg.switch_relay is not None:
                module_instance.switchpower(arg.switch_relay == 1)

    mqtt.configure_tls()
    try:
        mqtt.connect_and_loop(loop_forever=False)

        if arg.push_ota:
            if not os.path.exists(arg.push_ota):
                raise OSError(f'--push-ota: file \"{arg.push_ota}\" does not exists')
            ota_module.push_ota(arg.push_ota, 1)

        while True:
            sleep(0.1)

    except KeyboardInterrupt:
        mqtt.disconnect()
